/*

  Use for Research purpose only.
  If you make any modification, please send me details.

  Render Silhouette, Ridges, Valleys and Intersection edges for triangle soup
  No pre-processing
  No connectivity information required
  
    Code is based on the following paper 

	"Hardware Support for Non-photorealistic Rendering"
	Ramesh Raskar
	SIGGRAPH/EuroGraphics Graphics Hardware 2001, Los Angles


  
	Code by :
	Ramesh Raskar
	  raskar@cs.unc.edu
	  http://www.cs.unc.edu/~raskar
	  Mar 2001
	  
		
	Mouse buttons :
		  Left : rotate object
		  Middle : translate
		  Right : menu
	Keys
		  Use l,L, z, Z, s, S to change width of edges

  Note :

	The code is written primarily so that you can take parts of it
			and use in your code. For this purpose, the data-structures,
			file formats and user interface are kept very simple.
	The three functions you should look at are drawsilhouette### ().
			
*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <time.h>
#include <math.h>
#include "string.h"

#include "matrixFunc.h"
#include "otherStuff.h"


GLfloat nAngle = 90.0, nAngle2 = -90.0, nAngle3 = 0.0;
GLfloat xdist = 0, ydist = 0, zdist = 0.0;
int ox, oy;
int buttonState;


int CURRENT_MODEL = CUBE_MODEL;
GLfloat z_near = 1.12;
GLfloat lineWidth = 0.7;
GLfloat scaleT = 0.05;
int charcoalFlag = 0;
int colorObjFlag = 1;
GLfloat cosAngleThreshold = 0.0;

int rendermode = SIL_FAT; //initial mode

#define Res 1000
int winWidth = Res;
int winHeight = Res*3/4;

//GLfloat eye[3] = {100.0,100.0,-50.0};
GLfloat eye[3] = {0.0,0.0,0.0};
GLfloat light[3] = {0.7, 0.7, 0};

#include "creaseStuff.h"


////////////////////////////////////////////////////////////////////////
/* 
Method 1 for Silhouette (1999 paper):
WireFrame method
Draw front polygons filled
Draw back polygons black wireframe 
*/
void
drawsilhouetteLine()
{
	// Front
    if (colorObjFlag == 0) glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
    glCullFace(GL_BACK);
    glEnable(GL_CULL_FACE);
    glDepthMask(GL_TRUE);
    glColor3f(0.7f, 0.7f, 1.0f);
	glColor4f(1.0f, 1.0f, 0.f, 1.0f); 
    drawObject();
    
	// Back
    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
    glPolygonMode(GL_BACK, GL_LINE);
    glDepthFunc(GL_LEQUAL);
    glCullFace(GL_FRONT);
    glDepthMask(GL_FALSE);	/* depth buffer is read only */
    glColor3f(0.1f, 0.1f, 0.5f);
    glLineWidth(lineWidth);
	glPushMatrix();
	if (charcoalFlag == 0) drawObject(); else drawBackFattened();
	glPopMatrix();
	
	cleanGLstate();
}

/* 
Method 2 for Silhouette (1999 paper):
Translation of back facing polygons
Draw front polygons filled
Draw back polygons in black translated fwd towards the camera
*/

void
drawsilhouetteZscaled()
{
	// Front
    if (colorObjFlag == 0) glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
    glCullFace(GL_BACK);
    glEnable(GL_CULL_FACE);
    glDepthMask(GL_TRUE);
    glColor3f(0.7f, 0.9f, 1.0f);
	glColor4f(1.0f, 1.0f, 0.f, 1.0f); 
	drawObject();
	
	// Back
    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
    glPolygonMode(GL_BACK, GL_FILL);
    glDepthFunc(GL_LEQUAL);
    glCullFace(GL_FRONT);
    glDepthMask(GL_FALSE);	/* depth buffer is read only */
    glColor3f(0.2f, 0.2f, 0.5f);
	
	glPushMatrix();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( FOV,(float) winWidth/winHeight,z_near,ZFAR); /* create different z grading */
	glMatrixMode(GL_MODELVIEW);
	if (charcoalFlag == 0) drawObject(); else drawBackFattened();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(FOV, (float) winWidth/winHeight,1.0,ZFAR); /* create different z grading */
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	
	cleanGLstate();
}

/* 
Method 3 for Silhouette (1999 paper):
View dependent fattening of back facing polygons
Draw front polygons filled
Draw back polygons in black fattened
*/

void drawsilhouetteBackFattened()
{
	// Front
	if (colorObjFlag == 0) glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	glDepthMask(GL_TRUE);  
	//glColor4f(0.8f, 0.8f, 1.f, 1.0f); 
	glColor4f(1.0f, 1.0f, 0.f, 1.0f); 
	drawObject();
	
	// Back
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glPolygonMode(GL_BACK, GL_FILL);
	glDepthFunc(GL_LEQUAL);
	glDisable(GL_CULL_FACE);
	if (colorObjFlag == 0) glColor3f(0.5f, 0.5f, 1.0f); else glColor4f(0.f, 0.f, 0.f, 1.0f);
	
	glDisable(GL_BLEND);
	drawBackFattened();
	
	cleanGLstate();
}


// Demo how the program works without preprocessing by holding 'p' key
// Perturb the vertices
void perturb ()
{
	int i;
	int	num = CURRENT_MODEL;
	double x, y, z, shiftx, shifty, shiftz;
	//double fac = 1.0001*rand()/RAND_MAX;
	double fac = 0.001 + 0.0001*rand()/RAND_MAX;
	GLfloat normal[3];
	GLfloat temp1[3], temp2[3];


	for (i= 0 ; i < vertStruct[num].numVerts; i++) {
			x = vertStruct[num].verts[i].d[0]/300.0 - 0.5;
			y = vertStruct[num].verts[i].d[1]/300.0 - 0.5;
			z = vertStruct[num].verts[i].d[2]/300.0 - 0.5;
			shiftx = fac*( sin(10*y)*cos(11*x)*sin(z));
			shifty = fac*(cos(11*z)*sin(19*x));
			shiftz = fac*(sin(2*x));
			/*shiftx = fac*( -x*x + y*y + z*z);
			shifty = fac*(2*x + y*y - z);
			shiftz = fac*(x - y + z*z);*/
			vertStruct[num].verts[i].d[0] += shiftx*300.0;
			vertStruct[num].verts[i].d[1] += shifty*300.0;
			vertStruct[num].verts[i].d[2] += shiftz*300.0;
	}

	for (i= 0 ; i < trisStruct[num].numTris; i++) {
		//printf("i %d\n", i);
		// Assign normal and midpt
		vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d , 
			vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d, temp1) ;
		vecSub(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d ,
			vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d, temp2);
		vecCross(temp1, temp2, normal);
		vecNormalize(normal);
		vecAssign(normal, trisStruct[num].tris[i].norm);
		vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d ,
			vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d, temp1);
		vecAdd(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d ,temp1, temp2);
		vecScale(temp2, 1.0/3.0,trisStruct[num].tris[i].midpt);
		
	}
}
	

void drawObject()
{
	int i,num;
	num = CURRENT_MODEL;
	glDisable(GL_BLEND);
	glBegin(GL_TRIANGLES);
	for (i = 0; i < trisStruct[num].numTris; i ++) {
		if (num == BUNNY_MODEL) glColor3fv(trisStruct[num].tris[i].color);
		glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d);
		glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d);
		glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d);
	}
	glEnd();
	glEnable(GL_BLEND);
}

// drawObject but just front
void drawFront()
{
  int i,num;
  GLfloat temp[3];
  GLfloat normal[3], temp1[3], temp2[3];
  GLfloat eyeVec[3];
  GLfloat eyeT[3], lightT[3];
  GLfloat colval[3];
  GLfloat dotval, eyeDist, orntFactor, cosAngle;
  GLfloat modelview_mat[16], iview[16], id_mat[16];

  num = CURRENT_MODEL;
 
  glDisable(GL_BLEND);
 
  glGetFloatv(GL_MODELVIEW_MATRIX, modelview_mat);
  mat4Invert(modelview_mat, iview);
  mat4vec3Mult(iview, eye, eyeT);
   
   
  glBegin(GL_TRIANGLES);
  for (i = 0; i < trisStruct[num].numTris; i ++) {
	  vecSub( eyeT,trisStruct[num].tris[i].midpt, eyeVec);
	  cosAngle = vecDotProduct(eyeVec,trisStruct[num].tris[i].norm);
	  if (cosAngle <=0) continue; // remove backfacing
	  
	  glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d);
	  glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d);
	  glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d);
  }
  glEnd();
  
  glEnable(GL_BLEND);
}

// drawObject but just back
void drawBackSimple()
{

  int i,num;
  GLfloat eyeVec[3];
  GLfloat eyeT[3];
  GLfloat dotval, cosAngle;
  GLfloat modelview_mat[16], iview[16];

  num = CURRENT_MODEL;
 
  glDisable(GL_BLEND);
  glDisable(GL_CULL_FACE);
 
  //initShading();
  glGetFloatv(GL_MODELVIEW_MATRIX, modelview_mat);
  mat4Invert(modelview_mat, iview);
  mat4vec3Mult(iview, eye, eyeT);
   
   
  glColor3f(1.0,1.0,0.0);
  glBegin(GL_TRIANGLES);
  for (i = 0; i < trisStruct[num].numTris; i ++) {
	  vecSub( eyeT,trisStruct[num].tris[i].midpt, eyeVec);
	  cosAngle = vecDotProduct(eyeVec,trisStruct[num].tris[i].norm);
	  if (cosAngle >=0) continue; // remove frontfacing
	  glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[0]].d);
	  glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[1]].d);
	  glVertex3fv(vertStruct[num].verts[trisStruct[num].tris[i].vert[2]].d);
  }
  glEnd();
}


void menu(int mode)
{
	static i = 0;

	switch(mode) {
	case SIL_FAT:
	case OBJ_BACK:
	case OBJ_FRONT:
	case SIL_LINE:
	case SIL_ZSCALED:
	case OBJ_WIRED:
	case OBJ_WIREFILLED:
	case SHARP:
	case VALLEY:
	case SILRIDGE:
	case SILRIDGEVALLEY:
	case INTERSECT:
	case VALLEYINTERSECT:
		rendermode = mode;
		break;
	case SIL_AND_OBJ:
		colorObjFlag = 1 - colorObjFlag;
		break;
	case CHARCOAL:
		charcoalFlag = 1 - charcoalFlag;
		rendermode = SIL_FAT;
		printf("Charcoal Mode %d\n", charcoalFlag );
		break;
	case FACE_MODE :
		CURRENT_MODEL = FACE_MODEL;
		break;
	case VENUS_MODE :
		CURRENT_MODEL = VENUS_MODEL;
		break;
	case CUBE_MODE :
		CURRENT_MODEL = CUBE_MODEL;
		charcoalFlag = 0;
		break;
	case BUNNY_MODE :
		CURRENT_MODEL = BUNNY_MODEL;
		charcoalFlag = 0;
		break;
	}
	glutPostRedisplay();
}

void drawText(char *info_text)
{
	// printf("\n %s \n", info_text);
}


void
reshape(int wid, int ht)
{
	glViewport(0, 0, wid, ht);
	winWidth = wid;
	winHeight = ht;
	glutPostRedisplay();
}

int diffTime = 0, countTime = 0;

void
display()
{
	static int i = 0;
	//GLfloat light0pos[] = {0,0,0,1};
	int tstart, tstop;
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

	tstart = timeGetTime();
	

	glPushMatrix();

	// very simple user interface
	glTranslatef (xdist, ydist, zdist);	
	//glTranslatef(60,0,0);
    glRotatef ((GLfloat) nAngle, 0.0f, 0.0f, 1.0f);
    glRotatef ((GLfloat) nAngle2, 0.0f, 1.0f, 0.0f);
    glRotatef ((GLfloat) nAngle3, 1.0f, 0.0f, 0.0f);
    glRotatef (-90, 1.0f, 0.0f, 0.0f);
	
	switch(rendermode) {
	case SIL_LINE:
		drawText("Sil Line");
		drawsilhouetteLine();
		break;
	case SIL_ZSCALED:
		drawText("Sil Z Scaled");
		drawsilhouetteZscaled();
		break;
	case SIL_FAT:
		drawText("Silhouettes");
		drawsilhouetteBackFattened();
		if (sharpFlag) drawSharps();

		break;
	case SHARP : 
		drawText("Ridges");
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawSharps();
		break;
	case SILRIDGE :
		drawText("Silhouettes and Ridges");
		//sharpFlag = 1;
		drawsilhouetteBackFattened();
		drawSharps();
		//sharpFlag = 0;
		break;
	case SILRIDGEVALLEY :
		drawText("Sil, Ridges and Valleys");
		sharpFlag = 1;
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawValley2();
		sharpFlag = 0;
		break;
	case VALLEY :
		drawText("Valleys");
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawValley2();
		break;
	case INTERSECT :
		drawText("Intersection");
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawIntersect();
		break;
	case VALLEYINTERSECT : 
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(FOV, (float) winWidth/winHeight, 10.0, ZFAR);
		glMatrixMode(GL_MODELVIEW);
		drawValleyIntersect();
		break;
	case OBJ_BACK:
		glCullFace(GL_FRONT);
		glEnable(GL_CULL_FACE);
		glColor3f(0.8,0.8,1.0);
		charcoalFlag = 1; 
		drawBackFattened();
		charcoalFlag = 0; 
		break;
	case OBJ_FRONT:
		drawText("Traditional");
		glColor3f(0.8,0.8,1.0);
		glCullFace(GL_BACK);
		glEnable(GL_CULL_FACE);
		drawObject();
		break;
	case OBJ_WIRED:
		drawText("Wireframe");
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		glColor3f(0.0,0.0,0.0);
		glLineWidth(1.0);
		drawObject();
		cleanGLstate();
		break;
	case OBJ_WIREFILLED:
		drawText("Wireframe");
		glEnable(GL_CULL_FACE);
		glColor3f(0.99,0.99,1.0);
		glCullFace(GL_FRONT);
		glPolygonMode(GL_BACK, GL_FILL);
		drawObject();
		glColor3f(0.0,0.0,0.0);
		glCullFace(GL_BACK);
		glLineWidth(1.0);
		glPolygonMode(GL_FRONT, GL_LINE);
		drawObject();
		cleanGLstate();
		break;
	}
    glPopMatrix();
	glFinish();
	tstop = timeGetTime();
	
	countTime ++; diffTime += tstop - tstart;
    glutSwapBuffers();
	//printf("time = %d\n", tstop- tstart);
	if (countTime == 30) {  
		//printf("fps = %f\n", (float) 30*1000/diffTime); 
		countTime = 0; diffTime = 0;} 

}



void key(unsigned char key, int x, int y)
{
	switch(key) {
	case 'c' :
		charcoalFlag = 1 - charcoalFlag; 
		printf("Charcoal Mode %d\n", charcoalFlag);
		break;
	case 'z' :
		z_near += 0.01;	printf("z_near %f\n", z_near);
		break;
	case 'Z' :
		z_near -= 0.01;	printf("z_near %f\n", z_near);
		break;
	case 's' :
		scaleT += 0.01;	printf("scaleT %f\n", scaleT);
		break;
	case 'S' :
		scaleT -= 0.01;	printf("scaleT %f\n", scaleT);
		break;
	case 'l' :
		lineWidth += 0.1; printf("lineWidth %f\n", lineWidth);
		break;
	case 'L' :
		lineWidth -= 0.1; printf("lineWidth %f\n", lineWidth);
		break;
	case 't' :
		sharpThreshold += 0.01;
		printf("sharpThreshold %f\n", sharpThreshold*180.0/3.1415928);
		break;
	case 'T' :
		sharpThreshold -= 0.01;
		printf("sharpThreshold %f\n", sharpThreshold*180/3.1515928);
		break;
	case 'm' :
		sharpFlag  = 1 - sharpFlag;
		break;
	case ',' :
		zdist += 1;
		break;
	case '.' :
		zdist -= 1;
		break;
	case 'p' :
		perturb();printf("peturbed\n");
		break;
	case '\033': exit(0);
	}
    glutPostRedisplay();
}

// very simple user interface
void mouse(int button, int state, int x, int y)
{
	ox = x; oy = y;	buttonState = button;
}
void mousemotion(int x, int y)
{
	int dx, dy;
	dx = x - ox;
	dy = y - oy;
	
	switch ( buttonState)
	{
	case GLUT_LEFT_BUTTON :
		nAngle += 10*(float) dx/winWidth;
		nAngle2 += 10*(float) dy/winHeight;
		break;
	case GLUT_MIDDLE_BUTTON :
		xdist += 5*dx/40;
		//ydist -= 5*dy/40;
		zdist -= 5*dy/40;
		nAngle3 += 10*(float) dy/winHeight;
		break;
	}
    glutPostRedisplay();
}

void
main(int argc, char **argv)
{
	int val;

	printf("\n\n\nIMP : This program is *only* a demonstration using traditional API\n");
	printf("      Hence, performance of this version is likely to be low\n");
	printf("      But, there is no pre-processing and no connectivity information is used\n");
	printf("      During run time, processing involves only one polygon at a time\n\n");
	printf("      The proposed concept, when supported, is much more efficient\n\n");
	printf("\n\n (If the program complains about lack of Opengl32.dll\n");
	printf("   copy the file, Opengl32.generic.dll to Opengl32.dll in this directory)\n\n\n");

	printf("---Press enter to continue---\n"); //getchar();

    glutInit(&argc, argv);
    glutInitWindowSize(winWidth,winHeight);
//    glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE);
    glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE|GLUT_STENCIL);
//    glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_STENCIL);
    glutCreateWindow("(Click right mouse for menu; keys l,s,z,c,L,S,Z) Ramesh Raskar");
	//glutFullScreen();
    glutDisplayFunc(display);
    glutKeyboardFunc(key);
    glutReshapeFunc(reshape);
    glutMouseFunc(mouse);
    glutMotionFunc(mousemotion);
	glGetIntegerv(GL_STENCIL_BITS, &val);
	printf("\n --Your graphics support--\n\n OpenGL Version: %s, Renderer: %s,\n Vendor: %s, Stencil bits: %d\n", (char *) glGetString(GL_VERSION),
		glGetString(GL_RENDERER), glGetString(GL_VENDOR), val);
	
	if (val < 6) { printf("\n\n\n YOU DO NOT HAVE SUFFICIENT STENCIL BITS \n");
	printf("\n\n\n Valleys and Intersections will not be rendered correctly \n");}

    glutCreateMenu(menu);
    glutAddMenuEntry("This is a DEMO  raskar@merl.com", DEMO);
    glutAddMenuEntry("Object (first layer)", OBJ_FRONT);
    glutAddMenuEntry("Object (second layer)", OBJ_BACK);
    glutAddMenuEntry("Object as wireframe", OBJ_WIRED);
    glutAddMenuEntry("Object as wirefilled", OBJ_WIREFILLED);
    glutAddMenuEntry("1. Wireframe Mode", SIL_LINE);
    glutAddMenuEntry("2. Z scaled Translation", SIL_ZSCALED);
    //glutAddMenuEntry("3. Fattened Polygons", SIL_FAT);
    glutAddMenuEntry("Silhouettes", SIL_FAT);
    glutAddMenuEntry("Ridges", SHARP);
    glutAddMenuEntry("Valleys", VALLEY);
    glutAddMenuEntry("Intersect", INTERSECT);
    glutAddMenuEntry("Sil and Ridges", SILRIDGE);
    glutAddMenuEntry("Sil, Ridges and Valley", SILRIDGEVALLEY);
    glutAddMenuEntry("Valley and Intersect", VALLEYINTERSECT);
    glutAddMenuEntry("Toggle Object+Sil / Sil", SIL_AND_OBJ);
    glutAddMenuEntry("Toggle Charcoal Style", CHARCOAL);
	glutAddMenuEntry("Model - Face", FACE_MODE);
	glutAddMenuEntry("Model - Venus", VENUS_MODE);
	glutAddMenuEntry("Model - Jeep", CUBE_MODE);
	glutAddMenuEntry("Model - Bunny", BUNNY_MODE);
	glutAddMenuEntry("=Change threshold dihedral angle with keys T or t for ridges and valleys", -1);
	glutAddMenuEntry("=Change width using keys S,s,L,l,U,u", -1);
    glutAttachMenu(GLUT_RIGHT_BUTTON);
	
	#include "disable.h"
	
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
	
	
	// Read in three sample files	
    readModel("face.dat", FACE_MODEL);
	readModel("venus.dat", VENUS_MODEL);
	//readModel("funkyCube.dat", CUBE_MODEL);
	readModel("lrover.dat", CUBE_MODEL);
	//readModel("chair.dat", CUBE_MODEL);
	//readModel("simple2.dat", BUNNY_MODEL);
	//readModel("sparks.dat", BUNNY_MODEL);
	//readModel("alien.dat", BUNNY_MODEL);
	readModel("alienColor.dat", BUNNY_MODEL);
	
	printf("\n\nLeft mouse button for rotation\n");
	printf("\nMiddle mouse button for translation\n");
	printf("\nRight mouse button for menu\n\n");


    glClearColor(0.6,0.9,0.9,1.0);
    glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
    gluPerspective(FOV, (float) winWidth/winHeight,1.0,ZFAR);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
    glTranslatef(0,0,-500);
    glutMainLoop();
}


